/*
 * Copyright 2004-2009 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.doma.internal.jdbc.query;

import static org.seasar.doma.internal.util.AssertionUtil.*;

import java.sql.Statement;

import org.seasar.doma.internal.jdbc.entity.EntityPropertyType;
import org.seasar.doma.internal.jdbc.entity.EntityTypeFactory;
import org.seasar.doma.internal.jdbc.entity.GeneratedIdPropertyType;
import org.seasar.doma.internal.jdbc.sql.PreparedSqlBuilder;
import org.seasar.doma.internal.message.DomaMessageCode;
import org.seasar.doma.jdbc.JdbcException;
import org.seasar.doma.jdbc.id.IdGenerationConfig;

/**
 * @author taedium
 * 
 */
public class AutoInsertQuery<E> extends AutoModifyQuery<E> implements
        InsertQuery {

    protected boolean nullExcluded;

    protected GeneratedIdPropertyType<?> generatedIdPropertyType;

    protected IdGenerationConfig idGenerationConfig;

    public AutoInsertQuery(EntityTypeFactory<E> entityTypeFactory) {
        super(entityTypeFactory);
    }

    public void prepare() {
        assertNotNull(config, entityType, callerClassName, callerMethodName);
        executable = true;
        entityType.preInsert();
        prepareTable();
        prepareIdAndVersionProperties();
        prepareOptions();
        prepareTargetProperties();
        prepareVersionValue();
        prepareSql();
        assertNotNull(sql);
    }

    @Override
    protected void prepareIdAndVersionProperties() {
        super.prepareIdAndVersionProperties();
        generatedIdPropertyType = entityType.getGeneratedIdPropertyType();
        if (generatedIdPropertyType != null) {
            idGenerationConfig = new IdGenerationConfig(config, entityType,
                    tableName, generatedIdPropertyType.getColumnName());
            generatedIdPropertyType
                    .validateGenerationStrategy(idGenerationConfig);
            autoGeneratedKeysSupported = generatedIdPropertyType
                    .isAutoGeneratedKeysSupported(idGenerationConfig);
            generatedIdPropertyType.preInsert(idGenerationConfig);
        }
        versionPropertyType = entityType.getVersionPropertyType();
    }

    protected void prepareTargetProperties() {
        for (EntityPropertyType<?> p : entityType.getEntityPropertyTypes()) {
            if (!p.isInsertable()) {
                continue;
            }
            if (p.isId()) {
                if (p != generatedIdPropertyType
                        || generatedIdPropertyType
                                .isIncluded(idGenerationConfig)) {
                    targetProperties.add(p);
                }
                if (generatedIdPropertyType == null
                        && p.getWrapper().get() == null) {
                    throw new JdbcException(DomaMessageCode.DOMA2020,
                            entityType.getName(), p.getName());
                }
                continue;
            }
            if (p.isVersion()) {
                targetProperties.add(p);
                continue;
            }
            if (nullExcluded && p.getWrapper().get() == null) {
                continue;
            }
            if (!isTargetPropertyName(p.getName())) {
                continue;
            }
            targetProperties.add(p);
        }
    }

    protected void prepareVersionValue() {
        if (versionPropertyType != null) {
            versionPropertyType.setIfNecessary(1);
        }
    }

    protected void prepareSql() {
        PreparedSqlBuilder builder = new PreparedSqlBuilder(config);
        builder.appendSql("insert into ");
        builder.appendSql(tableName);
        builder.appendSql(" (");
        for (EntityPropertyType<?> p : targetProperties) {
            builder.appendSql(p.getColumnName());
            builder.appendSql(", ");
        }
        builder.cutBackSql(2);
        builder.appendSql(") values (");
        for (EntityPropertyType<?> p : targetProperties) {
            builder.appendWrapper(p.getWrapper());
            builder.appendSql(", ");
        }
        builder.cutBackSql(2);
        builder.appendSql(")");
        sql = builder.build();
    }

    @Override
    public void generateId(Statement statement) {
        if (generatedIdPropertyType != null) {
            generatedIdPropertyType.postInsert(idGenerationConfig, statement);
        }
    }

    public void setNullExcluded(boolean nullExcluded) {
        this.nullExcluded = nullExcluded;
    }

}
